package com.xy.service;

import com.xy.annotate.RestMappingController;
import com.xy.dto.BatchDeleteDto;
import com.xy.dto.SysRoleDto;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import com.xy.vo.RoleMenuVo;
import com.xy.vo.SysRoleGroupBySysVo;
import com.xy.vo.SysRoleVo;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/***
 * 系统角色服务类
 * @author 谭斌
 * @date 2022/12/12 16:37
 */
@RestMappingController("sysRole")
public interface SysRoleService {

    /**
     * 新增、更新角色信息
     *
     * @param sysRoleDto
     * @return
     */
    @ApiOperation(value = "新增、更新角色信息", notes = "新增、更新角色信息")
    @PostMapping("saveOrUpdate")
    R<Long> saveOrUpdate(@Valid @RequestBody SysRoleDto sysRoleDto);

    /**
     * 批量删除角色
     *
     * @param batchDeleteDto
     * @return
     */
    @ApiOperation(value = "删除角色", notes = "删除角色")
    @DeleteMapping
    R<Boolean> batchDelete(@Valid @RequestBody BatchDeleteDto batchDeleteDto);

    /**
     * 根据id获取角色信息
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "获取角色详情", notes = "获取角色详情")
    @GetMapping("{id}")
    R<SysRoleVo> getById(@PathVariable("id") Long id);

    /**
     * 分页查询角色信息
     *
     * @param page 分页对象 (current,size)
     * @param page 筛选参数 sysRoleVo
     * @return
     */
    @ApiOperation(value = "分页获取角色信息", notes = "分页获取角色信息")
    @GetMapping("page")
    R<PageBean<SysRoleVo>> getRolePage(PageBean page, SysRoleDto sysRoleDto);


    /**
     * 编辑角色菜单
     *
     * @param roleMenuVo
     * @return
     */
    @ApiOperation(value = "编辑角色菜单", notes = "编辑角色菜单")
    @PostMapping("saveRoleMenus")
    R<Boolean> saveRoleMenus(@RequestBody RoleMenuVo roleMenuVo);


    @ApiOperation(value = "删除角色", notes = "删除角色")
    @DeleteMapping("{id}")
    R<Boolean> deleteRoleById(@PathVariable("id") Long id);


    @ApiOperation(value = "根据系统获取角色列表", notes = "根据系统获取角色列表")
    @GetMapping("listBySys/{sysId}")
    R<List<SysRoleVo>> listBySysId(@PathVariable("sysId") Long sysId);

    @ApiOperation(value = "获取当前登录用户创建的角色列表", notes = "获取当前登录用户创建的角色列表")
    @GetMapping("myCreatedRoles}")
    R<List<SysRoleGroupBySysVo>> myCreatedRoles();
}
