package com.xy.utils;

import cn.dev33.satoken.stp.StpUtil;
import com.xy.collections.list.JArrayList;
import com.xy.collections.list.JList;
import com.xy.consts.CommConsts;
import com.xy.entity.SysMenuJoinSysRoleMenu;
import lombok.Data;
import lombok.experimental.Accessors;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Component
public class AuthorizeUtils {

    protected static RedisService<List<CacheEntity>> getRedisService() {
        return SpringBeanUtils.getBean(RedisService.class);
    }

    protected static String getKey(Long loginId) {
        return String.format("%s:%d", "menu", loginId);
    }

    /**
     * 获取菜单
     *
     * @param loginId  登录id
     * @param systemId 系统id
     * @return
     */
    public static List<SysMenuJoinSysRoleMenu> getMenu(Long loginId, Long systemId) {
        JList<SysMenuJoinSysRoleMenu> sysMenuJoinSysRoleMenus = new JArrayList<>();
        //获取redis缓存
        List<AuthorizeUtils.CacheEntity> cacheEntities = getRedisService().getMap(getKey(loginId), systemId.toString());
        if (!Emptys.check(cacheEntities)) {
            return sysMenuJoinSysRoleMenus;
        }
        //角色菜单累加
        cacheEntities.forEach(cacheEntity -> {
            if (Emptys.check(cacheEntity.getList())) {
                sysMenuJoinSysRoleMenus.addAll(cacheEntity.getList());
            }
        });
        //去重
        sysMenuJoinSysRoleMenus.comparing(SysMenuJoinSysRoleMenu::getId);
        return sysMenuJoinSysRoleMenus;
    }

    /**
     * 获取菜单
     *
     * @param systemId 系统id
     * @return
     */
    public static List<SysMenuJoinSysRoleMenu> getMenu(String token, Long systemId) {
        return getMenu(getLoginId(token, Long.class), systemId);
    }

    /**
     * 获取菜单树
     *
     * @param loginId  登录id
     * @param systemId 系统id
     * @return
     */
    public static List<SysMenuJoinSysRoleMenu> getMenuTree(Long loginId, Long systemId) {
        //获取菜单
        JList<SysMenuJoinSysRoleMenu> sysMenuJoinSysRoleMenus = new JArrayList<>(getMenu(loginId, systemId));
        //生成树
        JList<SysMenuJoinSysRoleMenu> tops = sysMenuJoinSysRoleMenus.filter().isNull(SysMenuJoinSysRoleMenu::getPaterId).list();
        for (SysMenuJoinSysRoleMenu top : tops) {
            List<SysMenuJoinSysRoleMenu> tree = Beans.tree(top, sysMenuJoinSysRoleMenus, SysMenuJoinSysRoleMenu::getId, SysMenuJoinSysRoleMenu::getPaterId, SysMenuJoinSysRoleMenu::getSonSysMenu);
            top.setSonSysMenu(tree);
        }
        return tops;
    }

    /**
     * 获取菜单树
     *
     * @param systemId 系统id
     * @return
     */
    public static List<SysMenuJoinSysRoleMenu> getMenuTree(String token, Long systemId) {
        return getMenuTree(getLoginId(token, Long.class), systemId);
    }

    /**
     * 判断数据查询权限
     *
     * @param loginId  登录id
     * @param systemId 系统id
     * @param systemId 接口uri
     * @return true=管理员数据查询权限 false=普通查询权限
     */
    public static boolean getDataAuth(Long loginId, Long systemId, String uri) {
        //获取redis缓存
        List<AuthorizeUtils.CacheEntity> cacheEntities = getRedisService().getMap(getKey(loginId), systemId.toString());
        if (!Emptys.check(cacheEntities)) {
            return false;
        }
        //遍历角色
        for (AuthorizeUtils.CacheEntity cacheEntity : cacheEntities) {
            //根据 接口uri和管理员数据查询权限=true 查找角色菜单，找到则有管理员数据查询权限，反之普通查询权限
            SysMenuJoinSysRoleMenu sysMenuJoinSysRoleMenu = new JArrayList<>(cacheEntity.getList()).filter()
                    .eq(SysMenuJoinSysRoleMenu::getInterfaceUri, uri)
                    .eq(SysMenuJoinSysRoleMenu::getSelectIsAdmin, true)
                    .object();
            if (sysMenuJoinSysRoleMenu != null) {
                return true;
            }
        }
        return false;
    }

    /**
     * 判断数据查询权限
     *
     * @param systemId 系统id
     * @param systemId 接口uri
     * @return true=管理员数据查询权限 false=普通查询权限
     */
    public static boolean getDataAuth(String token, Long systemId, String uri) {
        return getDataAuth(getLoginId(token, Long.class), systemId, uri);
    }

    /**
     * 获取所有系统ID
     *
     * @param loginId 登录id
     * @return
     */
    public static List<Long> getSystemIds(Long loginId) {
        Map<String, List<CacheEntity>> map = getRedisService().getMap(getKey(loginId));
        if (!Emptys.check(map)) {
            return new ArrayList<>();
        }
        List<Long> list = new ArrayList<>(map.size());
        map.forEach((s, cacheEntities) -> list.add(Long.parseLong(s)));
        return list;
    }

    /**
     * 获取所有系统ID
     *
     * @param token token
     * @return
     */
    public static List<Long> getSystemIds(String token) {
        return getSystemIds(getLoginId(token, Long.class));
    }

    /**
     * 检验系统ID是否属于当前登录人
     *
     * @param loginId  登录id
     * @param systemId 系统id
     * @return
     */
    public static Boolean checkSystemId(Long loginId, Long systemId) {
        Map<String, List<AuthorizeUtils.CacheEntity>> map = getRedisService().getMap(getKey(loginId));
        return map.containsKey(systemId.toString());
    }

    /**
     * 检验系统ID是否属于当前登录人
     *
     * @param token    token
     * @param systemId 系统id
     * @return
     */
    public static Boolean checkSystemId(String token, Long systemId) {
        return checkSystemId(getLoginId(token, Long.class), systemId);
    }

    /**
     * 获取所有角色ID
     *
     * @param loginId  登录id
     * @param systemId 系统id
     * @return
     */
    public static List<Long> getRoleIds(Long loginId, Long systemId) {
        List<AuthorizeUtils.CacheEntity> cacheEntities = getRedisService().getMap(getKey(loginId), systemId.toString());
        if (!Emptys.check(cacheEntities)) {
            return new ArrayList<>();
        }
        List<Long> list = new ArrayList<>(cacheEntities.size());
        cacheEntities.forEach(cacheEntity -> list.add(cacheEntity.getRoleId()));
        return list;
    }

    /**
     * 获取所有角色ID
     *
     * @param token    token
     * @param systemId 系统id
     * @return
     */
    public static List<Long> getRoleIds(String token, Long systemId) {
        return getRoleIds(getLoginId(token, Long.class), systemId);
    }

    /**
     * 获取所有角色code
     *
     * @param loginId  登录id
     * @param systemId 系统id
     * @return
     */
    public static List<String> getRoleCodes(Long loginId, Long systemId) {
        List<AuthorizeUtils.CacheEntity> cacheEntities = getRedisService().getMap(getKey(loginId), systemId.toString());
        if (!Emptys.check(cacheEntities)) {
            return new ArrayList<>();
        }
        List<String> list = new ArrayList<>(cacheEntities.size());
        cacheEntities.forEach(cacheEntity -> list.add(cacheEntity.getRoleCode()));
        return list;
    }

    /**
     * 获取所有角色code
     *
     * @param token    token
     * @param systemId 系统id
     * @return
     */
    public static List<String> getRoleCodes(String token, Long systemId) {
        return getRoleCodes(getLoginId(token, Long.class), systemId);
    }

    /**
     * 是否是管理员
     *
     * @param loginId  登录id
     * @param systemId 系统id
     * @return
     */
    public static Boolean isAdmin(Long loginId, Long systemId) {
        List<String> roleCodes = getRoleCodes(loginId, systemId);
        if (roleCodes.contains(CommConsts.ADMIN_ROLE_CODE)) {
            return true;
        }
        return false;
    }

    /**
     * 是否是管理员
     *
     * @param token    token
     * @param systemId 系统id
     * @return
     */
    public static Boolean isAdmin(String token, Long systemId) {
        return isAdmin(getLoginId(token, Long.class), systemId);
    }

    /**
     * 获取所有菜单接口uri
     *
     * @param loginId  登录id
     * @param systemId 系统id
     * @return
     */
    public static List<String> getMenuInterfaceUri(Long loginId, Long systemId) {
        JList<String> list = new JArrayList<>();
        //获取redis缓存
        List<AuthorizeUtils.CacheEntity> cacheEntities = getRedisService().getMap(getKey(loginId), systemId.toString());
        if (!Emptys.check(cacheEntities)) {
            return list;
        }
        //遍历角色
        for (AuthorizeUtils.CacheEntity cacheEntity : cacheEntities) {
            List<SysMenuJoinSysRoleMenu> sysMenuJoinSysRoleMenus = cacheEntity.getList();
            list.addAll(new JArrayList<>(sysMenuJoinSysRoleMenus).filter()
                    .isNotNull(SysMenuJoinSysRoleMenu::getInterfaceUri).list()
                    .getProperty(SysMenuJoinSysRoleMenu::getInterfaceUri)
            );
        }
        return list.comparing();
    }

    /**
     * 获取所有菜单接口uri
     *
     * @param token    token
     * @param systemId 系统id
     * @return
     */
    public static List<String> getMenuInterfaceUri(String token, Long systemId) {
        return getMenuInterfaceUri(getLoginId(token, Long.class), systemId);
    }

    /**
     * 登录鉴权
     *
     * @param token token
     * @return
     */
    public static boolean authByLogin(String token) {
        if (!Emptys.check(token)) {
            return false;
        }
        Object loginIdByToken = StpUtil.getLoginIdByToken(token);
        if (loginIdByToken == null) {
            return false;
        }
        return true;
    }

    /**
     * 接口鉴权
     *
     * @param token    token
     * @param systemId 系统id
     * @param uri      接口uri
     * @return
     */
    public static boolean authByInterface(String token, Long systemId, String uri) {
        //登录鉴权
        Boolean data = authByLogin(token);
        if (!data) {
            return false;
        }
        //接口鉴权
        List<String> menuInterfaceUri = getMenuInterfaceUri(getLoginId(token, Long.class), systemId);
        if (!menuInterfaceUri.contains(uri)) {
            return false;
        }
        return true;
    }

    /**
     * 数据鉴权
     *
     * @param token    token
     * @param systemId 系统id
     * @param uri      接口uri
     * @return
     */
    public static boolean authByData(String token, Long systemId, String uri) {
        //登录鉴权
        Boolean data = authByLogin(token);
        if (!data) {
            return false;
        }
        //数据鉴权
        long longId = getLoginId(token, Long.class);
        return getDataAuth(longId, systemId, uri);
    }

    /**
     * 获取登录id
     *
     * @param token  token
     * @param tClass id泛型
     * @param <T>
     * @return
     */
    public static <T> T getLoginId(String token, Class<T> tClass) {
        Object id = StpUtil.getLoginIdByToken(token);
        String idStr = id.toString();
        return tClass.equals(Long.class) ? (T) Long.valueOf(idStr)
                : tClass.equals(Integer.class) ? (T) Integer.valueOf(idStr)
                : tClass.equals(String.class) ? (T) idStr
                : (T) id;
    }

    @Data
    @Accessors(chain = true)
    public static class CacheEntity implements Serializable {

        /**
         * 角色id
         */
        private Long roleId;

        /**
         * 角色标识
         */
        private String roleCode;

        /**
         * 菜单集合
         */
        private List<SysMenuJoinSysRoleMenu> list;

    }
}
